@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
-# SPDX-License-Identifier: BSD-3-Clause
-# Copyright (c) 2023 Robin Jarry
+# SPDX-License-Identifier: Apache-2.0
+# Copyright (c) 2023-2024 Robin Jarry
 
 """
 Convert linux networking configuration to a DOT graph. The output can be piped
@@ -13,6 +13,7 @@ System dependencies: iproute2, ethtool
 
 import argparse
 import json
+import os
 import re
 import subprocess
 import sys
@@ -61,9 +62,6 @@ def main():
 
     namespaces = preprocess(get_running(), args)
 
-    print("// generated using netgraph")
-    print("// SPDX-License-Identifier: BSD-3-Clause")
-    print("// Copyright (c) 2023 Robin Jarry")
     print("graph {")
     print("  node [fontsize=11 fontname=monospace margin=0];")
     print("  edge [fontsize=11 fontname=monospace margin=0];")
@@ -73,7 +71,7 @@ def main():
         print()
         if ns:
             print(f"  subgraph {safe(ns)} {{")
-            print(f'    label="netns {ns}";')
+            print(f'    label="{ns}";')
             print("    cluster=true;")
             print()
             indent = "  "
@@ -149,9 +147,16 @@ def node_attrs(link):
     attrs["shape"] = SHAPES.get(link["kind"], "oval")
     if attrs["shape"] == "rectangle":
         attrs.setdefault("margin", "0.05")
-    labels = (
+    labels = [
+        f"<b>{link['name']}</b>",
+    ]
+    if "alias" in link:
+        alias = link["alias"]
+        if len(alias) > 16:
+            alias = alias[:15] + "…"
+        labels.append(f'<font color="slategray"><i>&quot;{alias}&quot;</i></font>')
+    labels += (
         [
-            f"<b>{link['name']}</b>",
             f'<font color="{color}">{link["kind"]}</font>',
         ]
         + [
@@ -177,7 +182,7 @@ def iproute2_cmd(prog, ns, *cmd):
     return json.loads(out)
 
 
-def ip(ns, *cmd):  # pylint: disable=invalid-name
+def ip(ns, *cmd):
     return iproute2_cmd("ip", ns, *cmd)
 
 
@@ -185,25 +190,58 @@ def bridge(ns, *cmd):
     return iproute2_cmd("bridge", ns, *cmd)
 
 
+def get_nsids(netns):
+    try:
+        missing_nsid = set(os.listdir("/run/netns"))
+    except OSError:
+        missing_nsid = set()
+    nsids = {}
+    for ns in ip(netns, "netns", "list-id"):
+        nsid = ns["nsid"]
+        if "name" in ns:
+            missing_nsid.discard(ns["name"])
+            nsids[nsid] = ns["name"]
+        elif nsid == 0:
+            # XXX: hack: there is absolutely no guarantee that 0 is a special
+            # number that will always point to the netns of PID 1. However,
+            # this workaround seems to work most of the time.
+            nsids[nsid] = ""
+    for name in missing_nsid:
+        cmd = ["ip"]
+        if netns != "":
+            cmd += ["-n", netns]
+        subprocess.call(
+            cmd + ["netns", "set", name, "auto"],
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+        )
+
+    if missing_nsid:
+        for ns in ip(netns, "netns", "list-id"):
+            nsid = ns["nsid"]
+            if nsid not in nsids and "name" in ns:
+                nsids[nsid] = ns["name"]
+
+    return nsids
+
+
 def get_running():
     addrs = ip("", "addr", "show")
-    nsids = ip("", "netns", "list-id")
     running = {
         "": {
             "addr_names": {a["ifname"]: a for a in addrs},
             "addr_ids": {a["ifindex"]: a for a in addrs},
-            "nsids": {n["nsid"]: n["name"] for n in nsids if "name" in n},
+            "nsids": get_nsids(""),
         },
     }
     for ns in running[""]["nsids"].values():
         if ns == "":
             continue
         addrs = ip(ns, "addr", "show")
-        nsids = ip(ns, "netns", "list-id")
         running[ns] = {
             "addr_names": {a["ifname"]: a for a in addrs},
             "addr_ids": {a["ifindex"]: a for a in addrs},
-            "nsids": {n["nsid"]: n.get("name", "") for n in nsids},
+            "nsids": get_nsids(ns),
         }
 
     return running
@@ -243,12 +281,16 @@ def preprocess(conf, args):
                 kind = "???"
 
             data = addr.get("linkinfo", {}).get("info_data", {})
-            if "link_netnsid" in addr and "link_index" in addr:
-                link_ns = nsconfig["nsids"][addr["link_netnsid"]]
-                link_dev = conf[link_ns]["addr_ids"][addr["link_index"]]["ifname"]
-            elif "link" in addr:
+            if "link" in addr:
                 link_ns = ns
                 link_dev = addr["link"]
+            elif "link_netnsid" in addr and "link_index" in addr:
+                link_ns = nsconfig["nsids"].get(addr["link_netnsid"])
+                link_ids = conf.get(link_ns, {}).get("addr_ids", {})
+                if addr["link_index"] in link_ids:
+                    link_dev = link_ids[addr["link_index"]]["ifname"]
+                else:
+                    link_dev = None
             else:
                 link_ns = ns
                 link_dev = addr.get("linkinfo", {}).get("info_data", {}).get("link")
@@ -282,16 +324,20 @@ def preprocess(conf, args):
                         else:
                             vlans.add(v["vlan"])
 
-            tip = [f"{k} {v}" for k, v in data.items()]
+            tip = []
+            if "ifalias" in addr:
+                tip.append(f"alias {addr['ifalias']}")
+            tip += [f"{k} {v}" for k, v in data.items()]
             l2 = []
-            if args.l2_addresses:
-                l2.append(addr["address"])
-            else:
-                tip.append(f"lladdr {addr['address']}")
+            if "address" in addr:
+                if args.l2_addresses:
+                    l2.append(addr["address"])
+                else:
+                    tip.append(f"lladdr {addr['address']}")
 
             ipv4 = []
             ipv6 = []
-            for a in addr["addr_info"]:
+            for a in addr.get("addr_info", []):
                 if a["family"] == "inet":
                     net = f"{a['local']}/{a['prefixlen']}"
                     if args.ipv4_addresses and (a["scope"] == "global" or args.local):
@@ -320,6 +366,8 @@ def preprocess(conf, args):
                 a["master"] = safe(f"{ns}_{addr['master']}")
             if link_dev is not None:
                 a["link"] = safe(f"{link_ns}_{link_dev}")
+            if "ifalias" in addr:
+                a["alias"] = addr["ifalias"]
 
             out.setdefault(ns, []).append(a)
 
